/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * abstract class for client messages. it represents
 * CLIENT => SERVER messages on protocol specification.
 * every client messages should derived from this class.
 * 
 * @author Young Jin Yoon
 * @see ThincMsg
 */
public abstract class ThincCMsg extends ThincMsg {

	
	public ThincCMsg(ThincHandler h, ThincClientCanvas c) {
		super(h,c);
	}
	/**
	 * messageSize()
	 * 
	 * returns the total messageSize to send.
	 * @return total messageSize refered from ThincMsgHandler class
	 */
    public abstract short messageSize();
    /**
     * putMessage()
     * 
     * puts messages to send to server.
     * @param b ByteBuffer object to send
     */
    public abstract void putMessage(ByteBuffer b);
    /**
     * getMsgType()
     * 
     * returns a type of the message.
     * @return type of message
     */
	public abstract byte getMsgType();
	
	/**
	 * headerSize()
	 * 
	 * returns the header size of the client message. different from
	 * messageSize() method, it only returns the header of total messages.
	 * If you want to send more than thinc header to the server, override
	 * this method and put the proper header size onto overriden method.
	 * @see ThincCMsgInitRequestVideoClientFormat
	 * @return header size of message
	 */
	public short headerSize() { return messageSize(); }
	
	/**
	 * getFlag()
	 * 
	 * get the flag option from client message. it is just for future development.
	 * set to 0 for now. If you want set the flag on client message, override
	 * this method and put the proper flag onto overriden method.
	 * @return flags of message
	 */
	public byte getFlag() { return 0;} //Because We do not have any flags in CMsg now!
}
