/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.nio.ByteBuffer;
/**
 * <code>ThincCMsgEventMouseButton</code> handles the mouse clicked / released event from canvas. 
 * it simply implements <code>MouseListener</code> interfaces provided from java, and it is added
 * as a MouseListener on ThincClientCanvas.
 * 
 * @author Young Jin Yoon
 *
 */
public final class ThincCMsgEventMouseButton extends ThincCMsgEvent implements MouseListener{

	public static final int MESSAGE_TYPE = ThincProto.T_EV_BUTTON;
	public static final int MESSAGE_SIZE = ThincProto.T_EV_BUTTON_SIZE;
	
	public static final int BUTTON_PRESSED = ThincProto.T_DOWN;
	public static final int BUTTON_RELEASED = ThincProto.T_UP;

	private byte pressed;
	private MouseEvent send;
    /**
     * creates ThincCMsgEventMouseButton object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgEventMouseButton (ThincMsgHandler h, ThincClientCanvas c)
	{
		super(h,c);
		pressed = BUTTON_RELEASED;
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

    /**
     * putMessage()
     * simply puts buttonMask information
     * @see ThincCMsg.putMessage()
     */
	@Override
	public void putMessage(ByteBuffer b) {
		byte buttonMask = (byte) send.getButton();
		if (pressed == BUTTON_PRESSED) buttonMask |= 128;
		b.put(buttonMask);
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}
	
	/**
	 * process()
	 * simply calls the send method on <code> ThincMsgHandler </code>
	 */
	@Override
	public void process ()throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on MouseMove",e);
		}
	}

	/* implemented methods from MouseListener interfaces below*/

	/**
	 * mousePressed()
	 * 
	 * sets the pressed as BUTTON_PRESSED and saves MouseEvent object
	 * into send variable. Then invokes process() method.
	 */
	public void mousePressed(MouseEvent arg0) {
		pressed = BUTTON_PRESSED;
		send = arg0;
		canvas.drawThincCursor(arg0.getX(), arg0.getY());
		try {
			process();
		} catch (ThincException e) {
			System.err.println("process: send on MouseButton");
			e.printStackTrace();
			//TODO fill it in. make setLogin(String reason) in ThincClientCanvas!
		}
	}

	/**
	 * mouseReleased()
	 * 
	 * sets the pressed as BUTTON_RELEASED and saves MouseEvent object
	 * into send variable. Then invokes process() method. 
	 */
	public void mouseReleased(MouseEvent arg0) {
		pressed = BUTTON_RELEASED;
		send = arg0;
		canvas.drawThincCursor(arg0.getX(), arg0.getY());
		try {
			process();
		} catch (ThincException e) {
			System.err.println("process: send on MouseButton");
			e.printStackTrace();
			// TODO fill it in. make setLogin(String reason) in ThincClientCanvas!
		}
	}
	public void mouseClicked(MouseEvent arg0) {}
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}

}
