/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * authentication messages. it contains the id and password information for server.
 * @author  Young Jin Yoon
 * @see  ThincCMsgInit
 */
public final class ThincCMsgInitAuth extends ThincCMsgInit {
	public static final int MESSAGE_TYPE = ThincProto2.T_CLIENT_AUTH;
	public static final int MESSAGE_SIZE = ThincProto.T_CAUTH_SIZE;
	ThincClient client;
	
    /**
     * creates ThincCMsgInitAuth object.
     * @param cl <code>ThincClient</code> to get the information such as ID and password
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgInitAuth(ThincClient cl, ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
		client = cl;
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

	/**
	 * putMessage()
	 * 
	 * puts length of id, length of password, id, and password, respectively.
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		String id = client.getId();
		String passwd = client.getPasswd();
		b.putShort((byte)id.length());
		b.putShort((byte)passwd.length());
		b.put(id.getBytes());
		b.put(passwd.getBytes());
	}

	@Override
	public short messageSize() {
		return (short) (client.getId().length()+client.getPasswd().length()+MESSAGE_SIZE);
	}
	@Override
	public void process() throws ThincException{
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: in Authentication",e);
		}
	}

}
