/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * requests the initial frame buffer data.
 * 
 * @author Young Jin Yoon
 * @see ThincCMsgInitRequest
 */
public final class ThincCMsgInitRequestFBData extends ThincCMsgInitRequest {

	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_FBDATA; 
	public static final int MESSAGE_SIZE = ThincProto2.T_REQ_FBDATA_SIZE;
	
    /**
     * creates ThincCMsgInitRequestFBData object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgInitRequestFBData(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * putMessage()
	 * 
	 * puts request type and 0 for unused field, respectively.
	 */	
	@Override
	public void putMessage(ByteBuffer b) {
		b.putShort((short)INIT_MESSAGE_TYPE);
		b.putShort((short)0);
	}

	@Override
	public short messageSize() {
		return (short) (MESSAGE_SIZE + super.MESSAGE_SIZE);
	}

	@Override
	public void process() throws ThincException {
		
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on FBData",e);
		}
		
	}

}
