/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * let the server knows that client supported format for video.
 * 
 * @author Young Jin Yoon
 * @see ThincCMsgInitRequest
 */
public final class ThincCMsgInitRequestVideoClientFormat extends ThincCMsgInitRequest {

	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_VIDEO_CLIENT_FMTS; 
	public static final int MESSAGE_SIZE = ThincProto2.T_REQ_VIDEO_CLIENT_FMTS_SIZE;
	public static final int FMT_MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_FORMAT_SIZE;
	/**
     * creates ThincCMsgInitRequestVideoClientFormat object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincCMsgInitRequestVideoClientFormat(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * putMessage()
	 * 
	 * puts request type, 0 for unused field,and list of supported format, respectively.
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		b.putShort((short)INIT_MESSAGE_TYPE);
		b.putShort((short)0);
		
		/* get available formats from ThincVideoFormat class. it was set by ThincSMsgInitReplyVideoServerFormat*/
		int[] formats = ThincVideoFormat.getFormat();
		/* put the number of formats */
		b.putInt(formats.length);
		
		/* put each format using for loop */
		for(int i=0; i<formats.length;i++) {
			b.putInt(formats[i]);
		}
	}

	@Override
	public short messageSize() {
		/* total message size should be init_request size + init_client_format size + format size * format number */
		return (short) (MESSAGE_SIZE + super.MESSAGE_SIZE+ ThincVideoFormat.getFormat().length * 4);
	}
	
	@Override
	public short headerSize() {
		/* headersize is only init_request size + init_client_format size. */
		return (short) (MESSAGE_SIZE + super.MESSAGE_SIZE);
	}
	
	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on VideoClientFMT",e);
		}
	}

}
