/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * <code>ThincCMsgSoundStarts</code> sends the init messages
 * for sound. because it operates different port from frame buffer
 * or video messages, it is as same as <b>T_CSTART</b> messages from
 * Protocol specification.
 * @author Young Jin Yoon
 * @see ThincCMsgSound
 */
public class ThincCMsgSoundStart extends ThincCMsgSound {
	public static final int MESSAGE_TYPE = ThincProto.T_CSTART;
	public static final int MESSAGE_SIZE = ThincProto.T_CSTART_SIZE;

	public static final int VERSION_LENGTH = 3;
	byte[] version;
	
    /**
     * creates ThincCMsgSoundStart object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param version version information from the sound handler
	 */
	public ThincCMsgSoundStart(ThincHandler h, byte[] version) throws ThincException {
		super(h);
		if(version.length != VERSION_LENGTH) throw new ThincException("ThincCMsgSoundStart: invalid version");
		else this.version = version;
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}

	/**
	 * putMessage()
	 * 
	 * just puts the version information with network order.
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		for(int i=0; i< version.length; i++)
			b.put(version[i]);
	}

	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on VideoStartOK",e);
		}

	}

}
