/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * <code>ThincCMsgVideoStartOK</code> sends an ACK/NAK messages
 * for video stream to the server.
 * 
 * @author Young Jin Yoon
 * @see ThincCMsgVideo
 * @deprecated server does not send the next message if it is sent.
 * if you want to use it, only use this when you want to stop video.
 */
public final class ThincCMsgVideoStartOK extends ThincCMsgVideo {
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_STARTOK_SIZE;
	public static final int MESSAGE_TYPE = ThincProtoVideo.T_VIDEO_STARTOK;
	public static final int OK = ThincProtoVideo.T_VIDEO_OK;
	public static final int NOTOK = ThincProtoVideo.T_VIDEO_NOTOK;
	private int id;
	private boolean ok;
	
	
    /**
     * creates ThincCMsgSoundStart object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 * @param id identifier for each video streams
	 * @param startOK boolean flag that refers OK or not OK
	 */
	public ThincCMsgVideoStartOK(ThincMsgHandler h, ThincClientCanvas c, int id, boolean startOK) {
		super(h, c);
		this.id = id;
	}

	@Override
	public byte getMsgType() {
		return MESSAGE_TYPE;
	}

	/**
	 * putMessage()
	 * 
	 * puts stream id and its ACK or NAK. 1 for ACK, 0 for NAK
	 */
	@Override
	public void putMessage(ByteBuffer b) {
		b.putInt(id);
		if(ok)b.put((byte)OK);
		else b.put((byte)NOTOK);
	}

	@Override
	public short messageSize() {
		return MESSAGE_SIZE;
	}

	@Override
	public void process() throws ThincException {
		try {
			handler.send(this);
		} catch (ThincException e) {
			throw new ThincException("process: send on VideoStartOK",e);
		}
	}

}
