/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * general thinc message class. every thinc message (including SERVER=>CLIENT and CLIENT=>SERVER message) should be derived from this.
 * @author  Young Jin Yoon
 * @see ThincSMsg
 * @see  ThincCMsg
 */
public abstract class ThincMsg {

	protected ThincHandler handler; // handler to send the message
    protected ThincClientCanvas canvas; // canvas to draw 
    
    public ThincMsg(ThincHandler h, ThincClientCanvas c){
    	handler = h;
    	canvas = c;
    }

    /**
     * process()
     * 
     * is general processing method for each thinc messages.
     * 
     * Most of client messages, it simply invokes send()
     * methods in <code>ThincMsgHandler</code> object to send the
     * client messages.
     * 
     * In server messages, it interacts with <code>ThincClientCanvas</code>
     * in this method.
     * @throws ThincException
     */
    public abstract void process() throws ThincException; 
}
