/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * THINC application sharing protocol specification, JAVA
 * @author Ricardo Baratto
 *
 */

	/* set tabstop = 8 */

	/* based on 
	 * 'Protocols for Application and Desktop Sharing'
	 * http://www1.cs.columbia.edu/~lennox/draft-lennox-avt-app-sharing-00.html
	 */

	/* header. It's not the same as defined by the draft document, as we want to
	 * support multiple applications with multiple windows on the same connection.
	 * The draft defines the header to be:
	 * x_off: 32bit
	 * y_off: 32bit
	 * win_id: 16bit
	 * mbz: 8bit
	 * pt: 8bit (payload type)
	 *
	 * x and y depend on the message type so we don't put them in the header.
	 * win_id is still 16 bits.
	 * pt: defined in thinc_header as the message type (see thincProto.h)
	 */
	/* XXX HACK ALERT XXX */
	/* to have minimum disruption, when application sharing is being used
	 * this message will immediately follow the thinc_header. When app
	 * sharing is not being used, then there's no change from before.
	 * In protocol v2, we have a way to negotiate this at connection time.
	 * In the old protocol, i wish you the best of luck.
	 */

public interface ThincProtoApp {
	
	public class thinc_appHeader {
		byte app_id;			//:8;
		byte mbz;				//:8;
		short win_id;			//:16;
	} 

	public static final int T_APPHEADER_SIZE = 4;

	/* Description of an application */
	public class thinc_appDesc {
		byte id;			//:8;	/* id of app. used for all comm */
		byte name_len;	//:8;	/* the length of the name 
						//   (without final \0) */
		short win_count;	//:16;	/* how many windows the app has */
		/* followed by name_len chars which are the name of the app */
	}

	public static final int T_APP_DESC_SIZE	= 4;

	/* the application id of the whole desktop. The rest of the ids are generated
	 * by the server */
	public static final int T_APP_ID_DESKTOP = 0;

}
