/**
 * common definition of THINC protocol specification, JAVA
 * @author Ricardo Baratto
 */

	/* set tabstop=8 */
public interface ThincRecordCommon {

	/* if you change, don't forget the / at the end. lives depend on it */
	public static final String T_REC_DEFAULT_DIR = "/var/cache/thinc/";
	public static final int T_REC_DEFAULT_DIR_LEN = ((T_REC_DEFAULT_DIR.length()) - 1);

	/* magic number for THINC files 'THIN' */
	public static final char T_REC_MAGIC[] = {'T', 'H', 'I', 'N'};
	public static final int T_REC_MAGIC_SZ = 4;
	
	public static final String T_REC_IDX_NAME = "record.idx";
	public static final String T_REC_SHOT_EXT = ".shot";
	public static final int T_REC_SHOT_EXT_LEN = 5;
	public static final String T_REC_CMD_EXT = ".cmd";
	public static final int T_REC_CMD_EXT_LEN = 4;

	/* length of a cmd/screenshot filename */
	public static final int T_REC_FILEID_LEN = 20;

	public class thincRecIndexHeader {
		/* TODO: what should we put here?
		 * probably:
		 * - fb dimensions
		 * - color depth
		 * - lsb or msb
		 * - last known command and screenshot files
		 *
		 * no need for:
		 * - first files (just look at first record)
		 */
		int			dummy;//:32;
	}

	public static final int T_REC_INDEX_HEADER_SZ = 4;

	/* a record of a screenshot for the index file */
	class thincRecShot {
		/* doesn't include file extension or dir */
		byte	file[] = new byte[T_REC_FILEID_LEN];
		int		fp;//:32;
	}

	public static final int T_REC_SHOT_SZ = (T_REC_FILEID_LEN + 4);

	/* a record of a command for the index file */
	class thincRecCmd {
		/* doesn't include file extension or dir */
		byte	file[] = new byte[T_REC_FILEID_LEN];
		int		fp;//:32;
	}

	public static final int T_REC_CMD_SZ = (T_REC_FILEID_LEN + 4);

	/* a record in the index file */
	class thincRecRecord {
		int		time;//:32;
		thincRecShot 	shot;
		thincRecCmd	cmd;
	}

	public static final int T_REC_RECORD_SZ	= (4 + T_REC_SHOT_SZ + T_REC_CMD_SZ);

	/* a record in the command file */
	class thincRecCmdHeader {
		int			time;//:32;
		int			size;//:32;
		byte 			type;//:8;
		byte 			flags;//:8;
		byte pad[] = new byte[2];
	}

	public static final int T_REC_CMD_HEADER_SZ	= 10;

	/* record showing what the next command file is */
	public static final int T_REC_CMD_TYPE_NEXTFILE	= 0x01;
	public static final int T_REC_CMD_NEXTFILE_SZ = T_REC_CMD_HEADER_SZ + T_REC_FILEID_LEN;

	/* a record in the screenshot file */
	class thincRecShotHeader {
		int			time;//:32;
		int			size;//:32;
		short		width;//:16;
		short		height;//:16;
		short		bpp;//:16;
		short		info;//:16;
	}

	public static final int T_REC_SHOT_HEADER_SZ = 16;

	/* some info flags */

	/* record showing what the next screenshot file is */
	public static final int T_REC_SHOT_TYPE_NEXTFILE = 0x0001;
	public static final int T_REC_SHOT_NEXTFILE_SZ = T_REC_SHOT_HEADER_SZ + T_REC_FILEID_LEN;

	/* screenshot is compressed */
	public static final int T_REC_SHOT_COMPRESSED = 0x0002;


}
