/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;


/**
 * takes proper action for ack message from server. currently there is nothing to do.
 * if it is positive ack, do nothing. however if it is negative ack, it should be properly
 * handled by this class.
 * @author Young Jin Yoon
 * @see ThincSMsg
 */
public final class ThincSMsgAck extends ThincSMsg {
	public static final int MESSAGE_TYPE = ThincProto.T_SACK;
	public static final int MESSAGE_SIZE = ThincProto.T_SACK_SIZE;
	/**
     * creates ThincSMsgAck object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
     */
	public ThincSMsgAck(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}	
	
	/**
	 * process()
	 * 
	 * if the received message is NAK, simply throws new ThincException.
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		byte ack;
		/* receive message */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgAck",e);
		}
		ack = bb.get();
		
		/* if NAK, throw new ThincException */
		if(ack == 0) throw new ThincException("process: in ThincSMsgAck, NAK received");
	}
}
