/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for copying the regions from frame buffer and 
 * move it into specified region in frame buffer.
 * @author Young Jin Yoon
 * @see ThincSMsgFB
 */
public final class ThincSMsgFBCopy extends ThincSMsgFB {
	public static final int MESSAGE_TYPE = ThincProto.T_FB_COPY;
	public static final int MESSAGE_SIZE = ThincProto.T_FB_COPY_SIZE;
    /**
     * creates ThincSMsgFBCopy object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgFBCopy(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}
	
	/**
	 * process()
	 * 
	 * get the source coodinate, destination coordinate, width and height respectively,
	 * then copy the region using copyRegion() method in ThincClientCanvas class.
	 * and set it into ThincClientCanvas object.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		short sx, sy, dx, dy, width, height;

		/* get header message and parse it */		
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("process: error on FBCopy",e);
		}
		
		/* get coordinates, width and height */
		sx = bb.getShort();
		sy = bb.getShort();
		dx = bb.getShort();
		dy = bb.getShort();
		width = bb.getShort();
		height = bb.getShort();

		/* if previous canvas is resized, calculate resized coordinates, width and height */
		if(ThincSMsgFlagResize.isResized()) { 
			sx = ThincSMsgFlagResize.getResizedX(sx);
			sy = ThincSMsgFlagResize.getResizedY(sy);
			dx = ThincSMsgFlagResize.getResizedX(dx);
			dy = ThincSMsgFlagResize.getResizedY(dy);
			width = ThincSMsgFlagResize.getResizedX(width);
			height = ThincSMsgFlagResize.getResizedY(height);
		}

		/* copy the region from frame buffer to new position of frame buffer */
		canvas.copyRegion(sx, sy, dx, dy, width, height);	
	}

}
