/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for filling the regions with color to the frame buffer.
 * @author Young Jin Yoon
 * @see ThincSMsgFB
 * @see ThincSMsgFBFillBilevel
 */
public final class ThincSMsgFBFillSolid extends ThincSMsgFB {
	public static final int MESSAGE_TYPE = ThincProto.T_FB_SFILL;
	public static final int MESSAGE_SIZE = ThincProto.T_FB_SFILL_SIZE;
	public static final int RECT_MESSAGE_SIZE = ThincProto.T_RECTREGION_SIZE;
	
	/**
     * creates ThincSMsgFBFillSolid object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgFBFillSolid(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * get the region to draw and color from network and try to draw a filled 
	 * rectangle using drawFillRect() method from <code>ThincClientCanvas</code> class.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		int color;
		short numRects;
		short x, y, width, height;

		/* get header message and parse it */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("process: error on FBSolid",e);
		}
		
		color = bb.getInt(); // color to draw
		numRects = bb.getShort(); // number of regions represented by thinc_rectRegion.

		/* get thinc_rectRegion and draw it using for loop */
		for(int i=0; i<numRects; i++)
		{
			try {
				bb = handler.getMessage(RECT_MESSAGE_SIZE);
			} catch (ThincException e) {
				throw new ThincException ("process: error on FBFillSolid",e);
			}
			x = bb.getShort(); // get x to draw
			y = bb.getShort(); // get y to draw
			width = bb.getShort(); // get width to draw
			height = bb.getShort(); // get height to draw

			/*draw the image using drawFillRect in ThincClientCanvas class */
			canvas.drawFillRect(x, y, width, height, color);
		}
	}
	
}
