/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * class for cache flag message. receives cache message and puts/gets the object to/from the cache by id from server. 
 * @author  Young Jin Yoon
 * @see ThincSMsgFlag
 * @see  ThincSMsgFBRawUpdate
 */
public final class ThincSMsgFlagCache extends ThincSMsgFlag {
	public static final int ADD_MESSAGE_SIZE = ThincProtoCache.T_FB_CACHE_ADD_SIZE;
	public static final int DATA_MESSAGE_SIZE = ThincProtoCache.T_FB_CACHE_DATA_SIZE;
	
	private ThincMsgHandler handler;
	private ThincClientCanvas canvas;

	private int id;
	private ByteBuffer hd;
	private int type;
	
	/**
     * creates ThincSMsgFlagCache object.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgFlagCache(ThincMsgHandler h, ThincClientCanvas c) {
		handler = h;
		canvas = c;
		id = 0;
	}
	/**
	 * setCacheId()
	 * receives the cache id from network and save it temporary.
	 * it will be used in addCache() and methods.
	 * @param type cache type defined in ThincClientCanvas
	 * @throws ThincException
	 */
	public void setCacheId(int type) throws ThincException {
		/* receive cache id from network */
		try {
			hd = handler.getMessage(ADD_MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("setCacheId: error on receving cache message",e);
		}
		id = hd.getInt(); // set the cache id
		this.type = type; // set the type of cache
	}
	/**
	 * addCache()
	 * actually adds the object into cache. the key value and type such as id and type 
	 * in this class should be already set by setCacheId() method.
	 * @param target the object to put
	 * @throws ThincException
	 */
	public void addCache(Object target)throws ThincException {
		try {
			canvas.setCache(target, id, type);
		} catch (ThincException e) {
			throw new ThincException("addCache: cannot add it into cache",e);
		}
	}
	/**
	 * getCache()
	 * gets the object from the cache. the key value(identifier) will be automatically
	 * received from this method.
	 * @param type type of the cache
	 * @return the object to get
	 * @throws ThincException
	 */
	public Object getCache(int type) throws ThincException
	{
		Object retVal;
		/* receive the key value(identifier) */
		try {
			hd = handler.getMessage(DATA_MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("cacheAdd: error on process",e);
		}
		id = hd.getInt(); // id of cache
		
		/* get the object from cache */
		try {
			retVal = canvas.getCache(id, type);
		} catch (ThincException e) {
			throw new ThincException("cacheAdd: cannot get the object from cache",e);
		}
		return retVal;
	}
}
