/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for init NAK replies from server.
 * @author Young Jin Yoon
 * @see ThincSMsgInit
 */
public final class ThincSMsgInitNotOK extends ThincSMsgInit {
	public static final int MESSAGE_TYPE = ThincProto2.T_SERV_NOTOK;
	public static final int MESSAGE_SIZE = ThincProto2.T_SERV_NOTOK_SIZE;

	/**
     * creates ThincSMsgInitNotOK object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgInitNotOK(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}
	/**
	 * process()
	 * receives the reason of nak and print it into user window.
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		int reason_len;
		char[] reason;

		/* receive the length of the reason */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: error on NotOK",e);
		}
		reason_len = bb.getInt();
		
		/* receive the reason */
		reason = new char[reason_len];
		try {
			bb = handler.getMessage(reason_len);
		} catch (ThincException e) {
			throw new ThincException("process: error on NotOK",e);
		}
		for(int i=0; i<reason_len; i++)
			reason[i] = (char) bb.get();
		
		/* throw the exception for that reason */
		throw new ThincException("from server: "+String.copyValueOf(reason));	
	}

}
