/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * class for init ACK replies from server.
 * @author Young Jin Yoon
 * @see ThincSMsgInit
 */
public final class ThincSMsgInitOK extends ThincSMsgInit {
	public static final int MESSAGE_TYPE = ThincProto2.T_SERV_OK;
	
	/**
     * creates ThincSMsgInitNotOK object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgInitOK(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}

	/**
	 * process()
	 * gets the last sent message from <code>ThincMsgHandler</code> and
	 * conducts the proper action for each sent message.
	 */
	@Override
	public void process() throws ThincException {
		byte lastSent = ((ThincMsgHandler)handler).getLastSent();
		/* for each sent message type => currently only ThincCMsgInitAuth class is used*/
		switch(lastSent) {
		case ThincCMsgInitAuth.MESSAGE_TYPE: // if ACK for auth, try to send first message for init reply
			try {
				((ThincMsgHandler)handler).getInitHandler().sendNextMessage();
			} catch (ThincException e) {
				throw new ThincException("process: in InitOK",e);
			}
			break;
		default:
			throw new ThincException("process: in InitOK, unknown ACK from server");
		}
	}
}
