/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * initialize the cache size for bitmap,pixmap and image caches.
 * @author Young Jin Yoon
 * @see ThincSMsgInitReply
 */
public final class ThincSMsgInitReplyCacheSize extends ThincSMsgInitReply {
	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_CACHESZ; 
	public static final int MESSAGE_SIZE = ThincProto2.T_REPLY_CACHESZ_SIZE;

	/**
     * creates ThincSMsgInitReplyCacheSize object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */
	public ThincSMsgInitReplyCacheSize(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}

	/**
	 * process()
	 * gets the information for caches and set it using createCache() method
	 * from <code>ThincClientCanvas</code> class.
	 */
	@Override
	public void process() throws ThincException {
		byte img, bit, pix;
		ByteBuffer bb;
		/* try to get and parse the message for cache */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: error on CacheSize",e);
		}
		img = bb.get(); // cache size for image
		bit = bb.get(); // cache size for bitmap
		pix = bb.get(); // cache size for pixmap
		bb.get();		// padding -> discard it.

		/* invokes createCache() method in ThincClientCanvas to create cache */
		canvas.createCache(img, bit, pix);
	}
}
