/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for receving video format from the server. the 
 * <code> ThincMsgInitReplyHandler</code> will send the 
 * <code> ThincCMsgInitRequestClientFormat </code> to notify 
 * the client supported video format.
 * @author Young Jin Yoon
 * @see ThincSMsgInitReply
 */
public final class ThincSMsgInitReplyVideoServerFormat extends ThincSMsgInitReply {
	public static final int INIT_MESSAGE_TYPE = ThincProto2.T_REQ_VIDEO_SERV_FMTS;
	public static final int MESSAGE_SIZE = ThincProto2.T_REPLY_VIDEO_SERV_FMTS_SIZE;
	public static final int FMT_MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_FORMAT_SIZE;
	
	/**
     * creates ThincSMsgInitReplyVideoServerFormat object.
     * @param tc <code>ThincClient</code> to get the information for handshaking.
     * @param h <code>ThincMsgHandler</code> to handle the message
     * @param cs <code>ThincClientCanvas</code> to get the information
	 */	
	public ThincSMsgInitReplyVideoServerFormat(ThincClient tc, ThincMsgHandler h, ThincClientCanvas cs) {
		super(tc, h, cs);
	}

	/**
	 * process()
	 * 
	 * receive every supported format from the server and use bitwise & operator
	 * to get client supported format correspond with server. finally, 
	 * <code> ThincMsgInitReplyHandler </code> sends
	 * <code> ThincCMsgInitRequestClientFormat </code> to notify the client
	 * supported video format by using sendNextMessage() method.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		int numFormat, format;
		/* receive the number of format information */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: error on VideoServerFormat",e);
		}
		numFormat = bb.getInt();

		/* receive the actual formats using for loop */
		for(int i=0; i<numFormat;i++) {
			try {
				bb = handler.getMessage(FMT_MESSAGE_SIZE);
			} catch (ThincException e) {
				throw new ThincException("process: error on VideoServerFormat",e);
			}
			format = bb.getInt();

			ThincVideoFormat.setFormat(format); //set the supported format into ThincVideoFormat.
				//ThincMsgInitReplyHandler will send ThincCMsgInitRequestClientFormat to the server.
		}
	}
}
