/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

import javax.sound.sampled.SourceDataLine;
/**
 * class for closing the sound stream.
 * @author Young Jin Yoon
 * @see ThincSMsgSound
 */
public final class ThincSMsgSoundClose extends ThincSMsgSound {
	public static final int MESSAGE_TYPE = ThincProtoSound.T_SND_CLOSE;
	public static final int MESSAGE_SIZE = ThincProtoSound.T_SND_CLOSE_SIZE;
	/**
     * creates ThincSMsgSoundClose object.
     * @param h <code>ThincSoundHandler</code> to handle the message
 	 */
	public ThincSMsgSoundClose(ThincSoundHandler h) {
		super(h);
	}
	
	/**
	 * process()
	 * 
	 * gets the stream id for sound and remove it from hash table.
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException{
		SourceDataLine src;
		ByteBuffer bb;
		byte id;
		/* receive the stream id for sound to close */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgSoundPlay");
		}
		id = bb.get();

		/* close it by simply removing from hash table */
		
		src = (SourceDataLine) streamId.get(id);
		if(src != null) {
			src.drain();
			src.close();
		}
		streamId.remove(id);
	}
	
}
