/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;

/**
 * class for closing the video stream.
 * @author Young Jin Yoon
 * @see ThincSMsgVideo
 */
public final class ThincSMsgVideoEnd extends ThincSMsgVideo {
	public static final int MESSAGE_TYPE = ThincProtoVideo.T_VIDEO_END;
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_END_SIZE;

	/**
     * creates ThincSMsgVideoEnd object.
     * @param h <code>ThincSoundHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
 	 */
	public ThincSMsgVideoEnd(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * gets the stream id from the server to close, then closes
	 * stream by removing stream information from the vector.
	 * 
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		ByteBuffer bb;
		int id;
		/* get the stream id from the network */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgVideoEnd",e);
		}
		id = bb.getInt();
		
		/* simply remove it from the vector */
		if(streamId.get(id) != null) streamId.remove(id);
	}

}
