/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
/**
 * class for resizing the video stream.
 * @author Young Jin Yoon
 * @see ThincSMsgVideo
 */
public final class ThincSMsgVideoResize extends ThincSMsgVideo {
	public static final int MESSAGE_TYPE = ThincProtoVideo.T_VIDEO_RESIZE;
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_RESIZE_SIZE;

	/**
     * creates ThincSMsgVideoResize object.
     * @param h <code>ThincSoundHandler</code> to handle the message
     * @param c <code>ThincClientCanvas</code> to get the information
 	 */
	public ThincSMsgVideoResize(ThincMsgHandler h, ThincClientCanvas c) {
		super(h, c);
	}

	/**
	 * process()
	 * 
	 * gets the stream id,width, height and new format information 
	 * from the server, then set them onto the stream information. 
	 * 
	 * @throws ThincException
	 */
	@Override
	public void process() throws ThincException {
		
		ByteBuffer bb;
		ThincVideoStream src;
		int id;
		short width, height;
		/* gets the stream id,width, height and new format information from the network */
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException("process: in ThincSMsgVideoResize");
		}
		id = bb.getInt(); // stream id
		width = bb.getShort(); // new width for stream
		height = bb.getShort(); // new height for stream

		/* get the stream information from the vector */
		src = (ThincVideoStream) streamId.get(id); 

		/* set width, height and format onto the stream information */
		src.width = width;
		src.height = height;
		if(src.format instanceof ThincVideoFormatUYVY) src.format = new ThincVideoFormatUYVY((ThincMsgHandler)handler, width, height);
		if(src.format instanceof ThincVideoFormatYUY2) src.format = new ThincVideoFormatYUY2((ThincMsgHandler)handler, width, height);
		if(src.format instanceof ThincVideoFormatYV12) src.format = new ThincVideoFormatYV12((ThincMsgHandler)handler, width, height);
	}
}
