/* Copyright 2006, 2007 Columbia University */
/* This file is part of THINC.

    THINC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    THINC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with THINC.  If not, see <http://www.gnu.org/licenses/>.
*/
import java.nio.ByteBuffer;
import java.nio.IntBuffer;

/**
 * class for YV12 video format used in <code>ThincSMsgVideo</code>. 
 * @author  Young Jin Yoon
 * @see ThincVideoFormat
 * @see  http://www.fourcc.org
 */
public class ThincVideoFormatYV12 extends ThincVideoFormat {
	public static final int MESSAGE_SIZE = ThincProtoVideo.T_VIDEO_INFO_YV12_SIZE;
	int yuv_yoffset;
	int yuv_uoffset;
	int yuv_voffset;
	int yuv_ypitch;
	int yuv_upitch;
	int yuv_vpitch;
	ThincMsgHandler handler;

	/**
     * creates ThincVideoFormatYV12 object. it also receives the
     * message format information from the server.
     * @param th <code>ThincMsgHandler</code> to handle the message
	 * @param height 
	 * @param width 
 	 */
	public ThincVideoFormatYV12(ThincMsgHandler th, short width, short height) throws ThincException{
		ByteBuffer bb;
		handler = th;
		try {
			bb = handler.getMessage(MESSAGE_SIZE);
		} catch (ThincException e) {
			throw new ThincException ("ThincVideoFormatYV12: cannot receive message format",e);
		}
		/* get offsets from the network */
		yuv_yoffset = bb.getInt();
		yuv_uoffset = bb.getInt();
		yuv_voffset = bb.getInt();

		/* get pitches from the network */
		yuv_ypitch = bb.getInt();
		yuv_upitch = bb.getInt();
		yuv_vpitch = bb.getInt();	
	}

	@Override
	public IntBuffer process(int width, int height, int size) throws ThincException{
		ByteBuffer bb;
		IntBuffer retVal = IntBuffer.allocate(width * height);
		byte y0 = 0, v0 = 0, u0 = 0;
		int rgb0;
		int yVal, vVal, uVal;
		/* receive entire YUV image from the network */
		try {
			bb = handler.getMessage(size);
		} catch (ThincException e) {
			throw new ThincException ("ThincVideoFormatYV12: cannot receive message format",e);
		}
		/* get each pixel and try to make ARGB pixel for each yuv*/ 
		for(int i=0; i<height; i++) {
			for(int j=0; j<width; j++) {
				yVal = yuv_yoffset + (i * yuv_ypitch + j);
				vVal = yuv_voffset + ((int)(i / 2) * (int)(yuv_vpitch)+ (int)(j / 2));
				uVal = yuv_uoffset + ((int)(i / 2) * (int)(yuv_upitch)+ (int)(j / 2));
				if(yVal < bb.limit()) 
					y0 = bb.get(yVal);
				if(vVal < bb.limit()) 
					v0 = bb.get(vVal);
				if(uVal < bb.limit()) 
					u0 = bb.get(uVal);
				rgb0 = yuvToRgb(y0, u0, v0);
				retVal.put(rgb0);
			}
		}
		retVal.flip();
		return retVal;	
	}
}
