/* Copyright 2005, 2007 Columbia University */
/*
 * tc_sound.c
 *
 * Sound handling functions for THINC client. Makes use of the alsa library.
 *
 * Author: Leo Kim
 * Network Computing Lab - Columbia University
 */
#ifndef _TC_SOUND_H_
#define _TC_SOUND_H_

#include <netinet/in.h>
#include <alsa/asoundlib.h>
#include "thincProtoSound.h"

/* set tabstop 8 */

/********************************* Constants *********************************/
/* this constant is added to the client's graphics port number to get the
 * sound port number */
#define TC_SND_PORT_INCR	10000

/* buffer jitter crap. the current values mean:
 * - in normal conditions we play 30 times per second (every 33ms)
 * - we have max 1 sec in the buffer
 * - we buffer about 66ms of data before playback starts
 */
#define TC_SND_CHUNKS_PER_SEC	30	/* how many times we want to play in
					   1 sec */
/* how many chunks can we fit in the buffer */
#define TC_SND_CHUNKS_IN_BUF	TC_SND_CHUNKS_PER_SEC
#if 0
#define TC_SND_CHUNKS_WAIT	2	/* how many chunks do we actually
					   buffer before starting to play */
#endif
#if 0
#define TC_SND_BUF_TIME		300	/* how much time we buffer before we
					   start playing (in ms) */
#else
#define TC_SND_BUF_TIME		200	/* how much time we buffer before we
					   start playing (in ms) */
#endif
#if 0
#define TC_SND_MIN_BUF_SIZE	1U<<16	/* absolute minimum buffer size */
#else
#define TC_SND_MIN_BUF_SIZE	1U<<13	/* absolute minimum buffer size */
#endif

/* capture stuff */
#if 0
#define TC_SND_CAP_BUF		(TC_SND_MIN_BUF_SIZE)*4
#define TC_SND_CAP_SIZE		(TC_SND_MIN_BUF_SIZE)
#else
#define TC_SND_CAP_SIZE		1U<<12
#define TC_SND_CAP_BUF		(TC_SND_CAP_SIZE)*8
#endif

/* for version checking */
#define TC_SND_VERSION1		0
#define TC_SND_VERSION2		1
#define TC_SND_VERSION3		1

/********************************* Data types *********************************/

/* because we use it for a prototype */
struct tc_fb;

/* for instrumentation */
struct tc_stamps;

struct tc_snd_params {
	unsigned int bits;
	unsigned int channels;
	unsigned int rate;
	unsigned int sample_size;
};

/* for handling sound. for now, we support only playing one type of stream at a
 * time. that is, if another sound stream comes in with different setup
 * parameters, then that would suck. */
struct tc_snd {
	/* network i/o */
	int sock;
	int usock;	/* udp. if 0, means we're doing TCP for everything */
	SSL *ssl;

	/* audio i/o */
	int play_sock;
	int cap_sock;

	short play_events;
	short cap_events;
 	/* our playback buffer */
	char *play_buf;
	char *play_end;	/* the end of the buffer */
	char *play_add;	/* where we add new data */
	char *play_play;	/* where we get data to play */

	unsigned long buf_size;		/* total size of the buffer in bytes */
	unsigned int num_chunks;
	unsigned long threshold;
	unsigned long curr_size;	/* how much we have in the buffer */
#if 0
	snd_pcm_uframes_t buf_size_s;	/* in samples */
	snd_pcm_uframes_t chunk_size;
#endif
	/* our capture buffer */
	char *cap_buf;
	char *cap_end;	/* end of the buffer */
	char *cap_add;	/* where to put newly captured data */
	char *cap_send;	/* where we get data from to send to tsd */

	unsigned long cap_bufsize;	/* total size of capture buffer in 
					   bytes */

	/* alsa handles */
	snd_pcm_t *pcm;
	snd_pcm_t *pcm_cap;

	/* UDP data */
	thinc_sndUDPData *udp_data;

	/* information about the active streams */
	struct tc_snd_params pparams;
	struct tc_snd_params cparams;

 	/* In the event that video had sync'ed with sound, we set this flag to
 	 * true. This way, we don't block forever while checking for more data
 	 * on the sound socket, since it's possible that video sync'ed with all
 	 * available sound. */
 	int didSync;
 	/* timestamp of current audio chunk for synchronization */
 	unsigned int timestamp;
	/* sockaddr for (re)trying to connect to tsd */
	struct sockaddr_in addr;
	/* for instrumentation */
	struct tc_stamps instr_stamps;
};

/********************************* Prototypes *********************************/
void tc_killSound(struct tc_fb *fb);
void tc_closeConnTsd(struct tc_fb *fb);
int tc_doConnectTsd(struct tc_fb *fb, int port);
int tc_handleSound(struct tc_fb *fb);
int tc_hasSoundData(struct tc_fb *fb);
int tc_initSound(struct tc_fb *fb);
int tc_soundCanPlay(struct tc_fb *fb);

#endif /* _TC_SOUND_H_ */
